//
//  MadMimiSubscribeAppDelegate.m
//  MadMimiSubscribe
//
//  Created by Justin Williams on 1/16/10.
//  Copyright 2010 Second Gear. All rights reserved.
//

#import "MadMimiSubscribeAppDelegate.h"
#import "SGMimiMailer.h"

static NSString * const kUserName = @"EMAIL_ADDRESS";
static NSString * const kApiKey = @"API_KEY";
static NSString * const kListName = @"LIST_NAME";

@interface MadMimiSubscribeAppDelegate ()
- (BOOL)validateEmailAddress:(NSString *)theAddress;
@end

@implementation MadMimiSubscribeAppDelegate

@synthesize window;
@synthesize emailTextField;
@synthesize cancelButton;
@synthesize joinButton;
@synthesize mailer;

- (void)applicationDidFinishLaunching:(NSNotification *)theNotification 
{
  self.mailer = [[SGMimiMailer alloc] initWithUserName:kUserName key:kApiKey delegate:self];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication
{
  return YES;
}

#pragma mark -
#pragma mark IBAction Methods
// +--------------------------------------------------------------------
// | IBAction Methods
// +--------------------------------------------------------------------

- (IBAction)cancel:(id)sender
{
  [self.window performClose:sender];
}

- (IBAction)join:(id)sender
{
  NSString *email = [self.emailTextField stringValue];
  BOOL isValidEmail = [self validateEmailAddress:email];
  
  if (isValidEmail)
  {
    NSError *error = nil;
    [self.mailer addEmailAddress:email toList:kListName error:&error];    
  }
  else 
  {
    NSRunAlertPanel(@"Malformed Email Address", @"That doesn't look to be a valid email address.  Check your spelling.", @"OK", nil, nil);
  }
}


#pragma mark -
#pragma mark SGMimiMailer Delegate Methods
// +--------------------------------------------------------------------
// | SGMimiMailer Delegate Methods
// +--------------------------------------------------------------------

- (void)requestSucceeded
{
  NSRunAlertPanel(@"Hey, it worked!", @"You're Subscribed!", @"Close", nil, nil);
  [self.window performClose:nil];
}

- (void)requestFailedWithError:(NSError *)theError
{
  if (theError != nil)
    NSRunAlertPanel(@"List Subscription Failed", [theError localizedDescription], @"Close", nil, nil);  
}

#pragma mark -
#pragma mark Class Extension Methods
// +--------------------------------------------------------------------
// | Class Extension Methods
// +--------------------------------------------------------------------

- (BOOL)validateEmailAddress:(NSString *)theAddress
{
  // RFC 2822
  // from http://www.regular-expressions.info/email.html
  //
  NSString *emailRegex = @"(?:[a-z0-9!#$%\\&'*+/=?\\^_`{|}~-]+(?:\\.[a-z0-9!#$%\\&'*+/=?\\^_`{|}"
  @"~-]+)*|\"(?:[\\x01-\\x08\\x0b\\x0c\\x0e-\\x1f\\x21\\x23-\\x5b\\x5d-\\"
  @"x7f]|\\\\[\\x01-\\x09\\x0b\\x0c\\x0e-\\x7f])*\")@(?:(?:[a-z0-9](?:[a-"
  @"z0-9-]*[a-z0-9])?\\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\\[(?:(?:25[0-5"
  @"]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-"
  @"9][0-9]?|[a-z0-9-]*[a-z0-9]:(?:[\\x01-\\x08\\x0b\\x0c\\x0e-\\x1f\\x21"
  @"-\\x5a\\x53-\\x7f]|\\\\[\\x01-\\x09\\x0b\\x0c\\x0e-\\x7f])+)\\])"; 
  NSPredicate *emailTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", emailRegex]; 
  
  return [emailTest evaluateWithObject:theAddress];
}

@end
