//
//  SGMimiMailer.m
//  SGMimiMailer
//
//  Created by Justin Williams on 1/17/10.
//  Copyright 2012 Second Gear. All rights reserved.
//

#import "SGMimiMailer.h"

@interface SGMimiMailer ()
@property (nonatomic, strong) NSString *userName;
@property (nonatomic, strong) NSString *apiKey;
@property (nonatomic, strong) NSMutableData *responseData;
@property (nonatomic, strong) NSURLConnection *connection;
- (BOOL)isValidDelegateForSelector:(SEL)theSelector;
@end

@implementation SGMimiMailer

@synthesize userName = _userName;
@synthesize apiKey = _apiKey;
@synthesize responseData = _responseData;
@synthesize delegate = _delegate;
@synthesize connection = _connection;


- (id)initWithUserName:(NSString *)userName key:(NSString *)apiKey delegate:(id <SGMimiMailerDelegate>)delegate
{
    if ((self = [super init]))
    {
        _userName = SG_RETAIN(userName);
        _apiKey = SG_RETAIN(apiKey);
        _delegate = delegate;
    }
    
    return self;
}

- (void)dealloc
{
    _delegate = nil;
    SG_RELEASE(_userName);
    SG_RELEASE(_apiKey);
    SG_RELEASE(_responseData);
    SG_RELEASE(_connection);
    SG_SUPER_DEALLOC;
}

#pragma mark -
#pragma mark Instance Methods
// +--------------------------------------------------------------------
// | Instance Methods
// +--------------------------------------------------------------------

- (BOOL)addEmailAddress:(NSString *)address toList:(NSString *)list error:(NSError **)error 
{
    NSString *urlString = [NSString stringWithFormat:@"http://api.madmimi.com/audience_lists/%@/add", list];
    NSURL *url = [[NSURL alloc] initWithString:urlString];
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] initWithURL:url];
    [request setHTTPMethod:@"POST"];
    SG_RELEASE(url);    
    
    NSString *paramString = [NSString stringWithFormat:@"username=%@&api_key=%@&email=%@", self.userName, self.apiKey, address];
    NSData *putParams = [paramString dataUsingEncoding:NSUTF8StringEncoding];
    [request setHTTPBody:putParams];
    
    self.connection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
    
    return YES;
}

#pragma mark -
#pragma mark NSURLConnectionDataDelegate Methods
// +--------------------------------------------------------------------
// | NSURLConnectionDataDelegate Methods
// +--------------------------------------------------------------------

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)theResponse 
{
    self.responseData = [[NSMutableData alloc] init];
    
    // Get response code.
    NSHTTPURLResponse *response = (NSHTTPURLResponse *)theResponse;
    NSInteger statusCode = [response statusCode];
    
    if ((statusCode != 200) && (statusCode != 304))
    {
        // Assume failure, and report to delegate.
        NSError *error = [NSError errorWithDomain:@"HTTP" code:statusCode userInfo:nil];
		if ([self isValidDelegateForSelector:@selector(requestFailedWithError:)])
        {
            [self.delegate requestFailedWithError:error];
        }
        
        [self.connection cancel];
    }
    else 
    {
        if ([self isValidDelegateForSelector:@selector(requestSucceeded)])
        {
			[self.delegate requestSucceeded];
        }
    }
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data 
{
    [self.responseData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error 
{
    if ([self isValidDelegateForSelector:@selector(requestFailedWithError:)])
    {
        [self.delegate requestFailedWithError:error];
    }
}

#pragma mark -
#pragma mark Class Extension Methods
// +--------------------------------------------------------------------
// | Class Extension Methods
// +--------------------------------------------------------------------

- (BOOL)isValidDelegateForSelector:(SEL)theSelector
{
	return ((self.delegate != nil) && ([self.delegate respondsToSelector:theSelector] == YES));
}

@end
